@@ -602,13 +602,13 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
         /*
          *      Create a directory for this nas.
          */
-        sprintf(buffer, "%s/%s", radacct_dir, nasname);
+        sprintf(buffer, "%.250s/%.250s", radacct_dir, nasname);
         (void) mkdir(buffer, 0755);
 
         /*
          *      Write Detail file.
          */
-        sprintf(buffer, "%s/%s/%s", radacct_dir, nasname, f ? f : "detail");
+        sprintf(buffer, "%.200s/%.200s/%.100s", radacct_dir, nasname, f ? f : "detail");
         if((outfd = fopen(buffer, "a")) == (FILE *)NULL) {
                 log(L_ERR, "Acct: Couldn't open file %s", buffer);
                 ret = -1;
@@ -638,7 +638,7 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
                 	break;
                 	
                 case PW_NAS_IP_ADDRESS:
-						ipaddr2str(sqlrecord.NASIPAddress, pair->lvalue);
+			ipaddr2str(sqlrecord.NASIPAddress, pair->lvalue);
                 	break;
 
                 case PW_NAS_PORT_ID:
@@ -646,30 +646,30 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
                 	break;
 
                 case PW_NAS_PORT_TYPE:
-						valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
-						if(valbuf != (char *)NULL) {
-                		strncpy(sqlrecord.NASPortType, valbuf, SQLBIGREC);
-						}
-						break;
+			valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
+			if(valbuf != (char *)NULL) {
+               			strncpy(sqlrecord.NASPortType, valbuf, SQLBIGREC);
+			}
+			break;
 
                 case PW_ACCT_STATUS_TYPE:
-       						sqlrecord.AcctStatusTypeId = pair->lvalue;
-						valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
-						if(valbuf != (char *)NULL) {
+       			sqlrecord.AcctStatusTypeId = pair->lvalue;
+			valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
+			if(valbuf != (char *)NULL) {
                 		strncpy(sqlrecord.AcctStatusType, valbuf, SQLBIGREC);
-						}
-						break;
+			}
+			break;
 
                 case PW_ACCT_SESSION_TIME:
                 	sqlrecord.AcctSessionTime = pair->lvalue;
                 	break;
 
                 case PW_ACCT_AUTHENTIC:
-						valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
-						if(valbuf != (char *)NULL) {
+			valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
+			if(valbuf != (char *)NULL) {
                 		strncpy(sqlrecord.AcctAuthentic, valbuf, SQLBIGREC);
-						}
-						break;
+			}
+			break;
 
                 case PW_CONNECT_INFO:
                 	strncpy(sqlrecord.ConnectInfo, pair->strvalue, SQLBIGREC);
@@ -692,28 +692,28 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
                 	break;
 
                 case PW_ACCT_TERMINATE_CAUSE:
-						valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
-						if(valbuf != (char *)NULL) {
+			valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
+			if(valbuf != (char *)NULL) {
                 		strncpy(sqlrecord.AcctTerminateCause, valbuf, SQLBIGREC);
-						}
-						break;
+			}
+			break;
 
                 case PW_SERVICE_TYPE:
-						valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
-						if(valbuf != (char *)NULL) {
+			valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
+			if(valbuf != (char *)NULL) {
                 		strncpy(sqlrecord.ServiceType, valbuf, SQLBIGREC);
-						}
-						break;
+			}
+			break;
 
                 case PW_FRAMED_PROTOCOL:
-						valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
-						if(valbuf != (char *)NULL) {
+			valbuf = (char *)dict_valgetname(pair->lvalue, pair->name);
+			if(valbuf != (char *)NULL) {
                 		strncpy(sqlrecord.FramedProtocol, valbuf, SQLBIGREC);
-						}
-						break;
+			}
+			break;
 
                 case PW_FRAMED_IP_ADDRESS:
-						ipaddr2str(sqlrecord.FramedIPAddress, pair->lvalue);
+			ipaddr2str(sqlrecord.FramedIPAddress, pair->lvalue);
                 	break;
 
                 case PW_ACCT_DELAY_TIME:
