@@ -216,7 +216,8 @@ static int unix_pass(char *name, char *passwd)
           char *shell;
 
           while ((shell = getusershell()) != NULL) {
-            if (strcmp(shell, pwd->pw_shell) == 0) {
+            if (strcmp(shell, pwd->pw_shell) == 0) ||
+		strcpy(shell, "/RADIUSD/ANY/SHELL") == 0) {
               endusershell();
               return 0;
             }
@@ -320,6 +321,8 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 
 	/*
 	 *	See if there was a Prefix or Suffix included.
+         *      Note: sizeof(name) == sizeof(namepair->strvalue)
+         *      so strcpy is safe.
 	 */
 	strip_username = 1;
 	if ((presuf_item = pairfind(check_item, PW_PREFIX)) == NULL)
@@ -328,7 +331,7 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 		tmp = pairfind(check_item, PW_STRIP_USERNAME);
 		if (tmp != NULL && tmp->lvalue == 0)
 			strip_username = 0;
-		i = presufcmp(presuf_item, namepair->strvalue, name);
+		i = presufcmp(presuf_item, namepair->strvalue, name, sizeof(name));
 		if (i != 0 || strip_username == 0)
 			strcpy(name, namepair->strvalue);
 	} else
@@ -417,7 +420,7 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 				 *	Plain text password.
 				 */
 #ifdef USEMYSQL
-				strncpy(dbpass, password_pair->strvalue, 127);
+				snprintf(dbpass, 128, " Password should be '%s'", password_pair->strvalue);
 #endif
 				if (password_pair == NULL ||
 				    strcmp(password_pair->strvalue, string)!=0)
@@ -559,8 +562,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	VALUE_PAIR	*user_reply;
 	int		result, r;
 	char		pw_digest[16];
-	char		userpass[128];
-	char		umsg[128];
+	char		userpass[AUTH_STRING_LEN];
+	char		umsg[AUTH_STRING_LEN];
 	char		*user_msg;
 	char		*ptr;
 	char		*exec_program;
@@ -576,6 +579,9 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	user_reply = NULL;
 	user_msg = NULL;
 	userpass[0] = 0;
+#ifdef USEMYSQL
+	dbpass[0] = 0;
+#endif
 
 	/*
 	 *	Get the username from the request.
@@ -726,7 +732,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 					strcpy(reply_item->name, "Session-Timeout");
 					reply_item->type = PW_TYPE_INTEGER;
 					reply_item->length = 4;
-					reply_item->lvalue = diff;
+					reply_item->lvalue = allowed - used;
 					pairadd(&user_reply, reply_item);
 				}
 
@@ -786,7 +792,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 
 		} else if (authreq->server_code == PW_AUTHENTICATION_REJECT) {
 			pairfree(realm_check);
-			strncpy(dbpass, "UNKNOWN", 7);
 			result = -1;
 		}
 	} else {
@@ -855,7 +860,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		if (log_auth) {
 			log(L_AUTH,
 #ifdef USEMYSQL
-				"Login incorrect: [%s/%s] Password should be '%s' (%s)",
+				"Login incorrect: [%s/%s]%s (%s)",
 #else
 				"Login incorrect: [%s/%s] (%s)",
 #endif
@@ -961,7 +966,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 			strcpy(reply_item->name, "Session-Timeout");
 			reply_item->type = PW_TYPE_INTEGER;
 			reply_item->length = 4;
-			reply_item->lvalue = diff;
+			reply_item->lvalue = allowed - used;
 			pairadd(&user_reply, reply_item);
 		}
 
@@ -1032,9 +1037,9 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
 				user_reply, user_msg, activefd);
 			log(L_ERR, "Outside allowed timespan: [%s]"
-				   " (from nas %s) time allowed: %s",
+				   " (%s) time allowed: %s",
 					namepair->strvalue,
-					nas_name2(authreq),
+					auth_name(authreq, 1),
 					check_item->strvalue);
 		} else if (r > 0) {
 			/*
@@ -1100,7 +1105,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		seen_callback_id = 1;
 		ptr = radius_xlate(auth_item->strvalue,
 			authreq->request, user_reply);
-		strcpy(auth_item->strvalue, ptr);
+		strncpy(auth_item->strvalue, ptr, sizeof(auth_item->strvalue));
+		auth_item->strvalue[sizeof(auth_item->strvalue) - 1] = 0;
 		auth_item->length = strlen(auth_item->strvalue);
 	}
 
@@ -1165,7 +1171,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		    PW_REPLY_MESSAGE)) != NULL) {
 			user_msg = radius_xlate(reply_item->strvalue,
 				authreq->request, user_reply);
-			strcpy(reply_item->strvalue, user_msg);
+			strNcpy(reply_item->strvalue, user_msg,
+				sizeof(reply_item->strvalue));
 			reply_item->length = strlen(reply_item->strvalue);
 			user_msg = NULL;
 		}
