@@ -10,19 +10,16 @@
 #
 # Returns:	0 = no duplicate, 1 = duplicate, >1 = error.
 #
-# Version:	@(#)checkrad.pl  1.22  08-Mai-2000  miquels@cistron.nl
+# Version:	@(#)checkrad.pl  1.18  25-Jul-1999  miquels@cistron.nl
 #
 #		livingston_snmp  1.1	Author: miquels@cistron.nl
 #		portslave_finger 1.0	Author: miquels@cistron.nl
 #		max40xx_finger	 1.0	Author: costa@mdi.ca
-#		max40xx_snmp  	 1.0	Author: blaz@gold.amis.net
-#		ascend_snmp      1.1    Author: blaz@amis.net
 #		computone_finger 1.1	Author: costa@mdi.ca
 #		sub tc_tccheck	 1.1	Author: alexisv@compass.com.ph
-#		cyclades_telnet	 1.2	Author: accdias@sst.com.br
+#		pathras_telnet	 1.1	Author: accdias@sst.com.br
 #		patton_snmp	 1.0	Author: accdias@sst.com.br
-#		digitro_rusers	 1.1	Author: accdias@sst.com.br
-#		cyclades_snmp	 1.0	Author: accdias@sst.com.br
+#		digitro_rusers	 1.0	Author: accdias@sst.com.br
 #		usrhiper_snmp    1.0    Author: igor@ipass.net
 #		multitech_snmp   1.0    Author: ehonzay@willmar.com
 #		netserver_telnet 1.0	Author: mts@interplanet.es
@@ -30,8 +27,6 @@
 #
 #	Config: $debug is the file you want to put debug messages in
 #		$snmpget is the location of your ``snmpget'' program
-#		$snmpwalk is the location of your ``snmpwalk'' program
-#		$rusers is the location of your ``rusers'' program
 #		$naspass is the location of your NAS admin password file
 #
 
@@ -45,9 +40,8 @@ $debug		= "";
 #$debug		= "$logdir/checkrad.log";
 
 $snmpget	= "/usr/bin/snmpget";
-$snmpwalk	= "/usr/bin/snmpwalk";
-$rusers		= "/usr/bin/rusers";
 $naspass	= "$raddbdir/naspasswd";
+$rusers		= "/usr/bin/rusers";
 
 #
 #	PM3:	$lv_offs is where the last S port is before one or two
@@ -338,55 +332,6 @@ sub max40xx_finger {
 	close FD;
 }
 
-#
-#	Check an Ascend Max4000 or similar model via SNMP
-#
-#	Note: Not all software revisions support finger
-#	      You may also need to enable the finger option.
-#
-#	Author: Blaz Zupan <blaz@gold.amis.net>
-#
-$asm	 = '.iso.org.dod.internet.private.enterprises.529';
-sub max40xx_snmp {
-	$sess_id = hex $ARGV[4];
-	$login = snmpget($ARGV[1], $ARGV[5], "$asm.12.3.1.4.$sess_id");
-
-	print LOG "  user at port S$ARGV[2]: $login\n" if ($debug);
-
-	($login eq $ARGV[3]) ? 1 : 0;
-}
-
-#
-#	Check an Ascend Max4000 or similar model via SNMP
-#
-#	Author: Blaz Zupan of Medinet <blaz@amis.net>
-#
-$asm   = '.iso.org.dod.internet.private.enterprises.529';
-sub ascend_snmp {
-	my $sess_id;
-	my $l1, $l2;
-
-	$l1 = '';
-	$l2 = '';
-
-	#
-	#	If it looks like hex, only try it as hex,
-	#	otherwise try it as both decimal and hex.
-	#
-	$sess_id = $ARGV[4];
-	if ($sess_id !~ /^0/ && $sess_id !~ /[a-f]/i) {
-		$l1 = snmpget($ARGV[1], $ARGV[5], "$asm.12.3.1.4.$sess_id");
-	}
-	$sess_id = hex $ARGV[4];
-	$l2 = snmpget($ARGV[1], $ARGV[5], "$asm.12.3.1.4.$sess_id");
-
-	print LOG "  user at port S$ARGV[2]: $l1 (dec)\n" if ($debug && $l1);
-	print LOG "  user at port S$ARGV[2]: $l2 (hex)\n" if ($debug && $l2);
-
-	(($l1 && $l1 eq $ARGV[3]) || ($l2 && $l2 eq $ARGV[3])) ? 1 : 0;
-}
-  
-
 #
 #	See if the user is logged in using the portslave finger.
 #
@@ -560,123 +505,113 @@ sub tc_tccheck {
 #
 #	Check a Cyclades PathRAS via telnet
 #
-#	Version: 1.2
+#	Version: 1.1
 #
 #	Author: Antonio Dias of SST Internet <accdias@sst.com.br>
 #
-sub cyclades_telnet {
-	#
-	#	Localize all variables first.
-	#
-	my ($pr, $pr_login, $pr_passwd, $pr_prompt, $endlist, @list, $port, $user);
-	#
-	#	This variable must match PathRAS' command prompt 
-	#	string as entered in menu option 6.2.
-	#	The value below matches the default command prompt.
-	#
-	$pr_prompt = '/Select option ==\>$/i';
+sub pathras_telnet {
+   #
+   #	Localize all variables first.
+   #
+   my (@users, $pathras, $port, $errmsg, $user, $login, $password);
+   #
+   #	Do we have Net::Telnet installed?
+   #
+   return 2 unless (check_net_telnet());
+   #
+   #	Get login name and password for NAS from $naspass file.
+   #
+   ($login, $password) = naspasswd($ARGV[1], 1);
+   #
+   #	NAS doesn't have a password defined
+   #
+   return 2 if ($password eq "");
+   #
+   #	Communicate with PathRAS using Net::Telnet, then access
+   #	menu option 6.8 to see who are logged in. This assumes 
+   #	you are using the default Prompt String (menu option 6.2)
+   #	and is based on PathRAS firmware version 1.1.9g
+   # 
+   #
+   $pathras = new Net::Telnet (
+   	Timeout => 10, 
+   	Prompt => '/(.*)\s:*$/', 
+   	Errmode => 'return'
+   );
+   #
+   #
+   #
+   if (!$pathras->open($ARGV[1])) {;
+      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
+      $pathras->close;
+      return 2;
+   }
+   #
+   #	Dunno why but PathRAS only issue its banner and 
+   #	the login prompt after receive a "CR" 
+   #
+   if (!$pathras->print("")) {
+      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
+      $pathras->close;
+      return 2;
 
-	#
-	#	This variable match the end of userslist.
-	#
-	$endlist = '/Type \<enter\>/i';
-
-	#
-	#	Do we have Net::Telnet installed?
-	#
-	return 2 unless (check_net_telnet());
-
-	#
-	#	Get login name and password for NAS 
-	#	from $naspass file.
-	#
-	($pr_login, $pr_passwd) = naspasswd($ARGV[1], 1);
-
-	#
-	#	Communicate with PathRAS using Net::Telnet, then access
-	#	menu option 6.8 to see who are logged in.
-	#	Based on PathRAS firmware version 1.2.3
-	#
-	$pr = new Net::Telnet (
-		Timeout		=> 10,
-		Host		=> $ARGV[1],
-		ErrMode		=> 'return'
-	) || return 2;
-
-	#
-	#	Force PathRAS shows its banner.
-	#
-	$pr->break();
-
-	#
-	#	Log on PathRAS
-	#
-	if ($pr->waitfor(Match => '/login : $/i') == 1) {
-		$pr->print($pr_login);
-	} else { 
-		print LOG " Error: sending login name to PathRAS\n" if ($debug);
-		$pr->close;
-		return 2;	
-	}
-
-	if ($pr->waitfor(Match => '/password : $/i') == 1) {
-		$pr->print($pr_passwd);
-	} else { 
-		print LOG " Error: sending password to PathRAS.\n" if ($debug);
-		$pr->close;
-		return 2;	
-	}
-
-	$pr->print();
-
-	#
-	#	Access menu option 6 "PathRAS Management"
-	#
-	if ($pr->waitfor(Match => $pr_prompt) == 1) {
-		$pr->print('6');
-	} else { 
-		print LOG "  Error: acessing menu option '6'.\n" if ($debug);
-		$pr->close;
-		return 2;
-	}
-	#
-	#	Access menu option 8 "Show Active Ports"
-	#
-	if ($pr->waitfor(Match => $pr_prompt) == 1) {
-		@list = $pr->cmd(String => '8', Prompt => $endlist);
-	} else { 
-		print LOG "  Error: acessing menu option '8'.\n" if ($debug);
-		$pr->close;
-		return 2;
-	}
-	#
-	#	Since we got the info we want, let's close 
-	#	the telnet session
-	#
-	$pr->close;
-
-	#
-	#	Lets examine the userlist stored in @list
-	#
-	foreach(@list) {
-		#
-		#	We are interested in active sessions only
-		#
-		if (/Active/i) {
-			($port, $user) = split;
-			#
-			#	Strip out any prefix, suffix and
-			#	realm from $user check to see if
-			#	$ARGV[3] matches.
-			#
-			if($ARGV[3] eq strip_username($user)) {
-				print LOG "  User '$ARGV[3]' found on '$ARGV[1]:$port'.\n" if ($debug);
-				return 1;
-			}
-		}
-	}
-	print LOG "  User '$ARGV[3]' not found on '$ARGV[1]'.\n" if ($debug);
-   	0;
+   }
+   #
+   #	Log on PathRAS
+   #
+   if (!($pathras->login($login, $password))) {
+      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
+      $pathras->close;
+      return 2;
+   }
+   #
+   #  Access menu option 6 "PathRAS Management"
+   #
+   if (!($pathras->cmd(String => '6', 
+   	Prompt => '/ ==\> $/', 
+   	Timeout => 10))) {
+      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
+      $pathras->close;
+      return 2;
+   }
+   #
+   #  Access menu option 8 "Show Active Ports"
+   #
+   if (!(@users = $pathras->cmd(String => '8', 
+   	Prompt => '/Type \<enter\>/', 
+   	Timeout => 10))) {
+      print LOG "  Error: " . $pathras->errmsg . "\n" if ($debug);
+      $pathras->close;
+      return 2;
+   }
+   #
+   # Since we got the info we want, let's close the session
+   #
+   $pathras->close;
+   #
+   #	Telnet closed.  We got the info so let's examine it.
+   #
+   foreach(@users) {
+      if (m/PPP Active|SLIP Active/) {
+         ($port, $user) = split;
+         #
+         # Strip out any prefixes and suffixes 
+         # from the username
+         #
+         $user = strip_username($user);
+         #
+         # Check if $user is already connected
+         #
+         if ($user eq $ARGV[3]) {
+            print LOG "  User $ARGV[3] found on " . 
+               "NAS $ARGV[1] port $port.\n" if ($debug);
+            return 1;
+         }
+      }
+   }
+   print LOG "  User $ARGV[3] not found on " .
+   	"NAS $ARGV[1].\n" if ($debug);
+   0;
 }
 
 #
@@ -693,7 +628,7 @@ sub patton_snmp {
    #
    # Check if the session still active
    # 
-   if (snmpget($ARGV[1], $ARGV[5], "$oid") == 0) {
+   if (snmpget($ARGV[1], "monitor", "$oid") == 0) {
       print LOG "  Session $ARGV[4] still active on NAS " . 
       	"$ARGV[1], port $ARGV[2], for user $ARGV[3].\n" if ($debug);
       return 1;
@@ -704,54 +639,26 @@ sub patton_snmp {
 #
 #      Check a Digitro BXS via rusers
 #
-#      Version: 1.1
+#      Version: 1.0
 #
 #      Author: Antonio Dias of SST Internet <accdias@sst.com.br>
 #
 sub digitro_rusers {
-   my ($ret);
-   local $_;
-
    if (-e $rusers && -x $rusers) {
-      #
-      # Get a list of users logged in via rusers
-      #
-      $_ =`$rusers $ARGV[1]`;
-      $ret = ((/$ARGV[3]/) ? 1 : 0);
+      open(FD, "$rusers $ARGV[1]|");
    } else {
       print LOG "   Error: can't execute $rusers\n" if $debug;
-      $ret = 2;
+      return 2;
    }
-   $ret;
-}
 
-#
-#	Check Cyclades PR3000 and PR4000 via SNMP
-#
-#	Version: 1.0
-#
-#	Author: Antonio Dias of SST Internet <accdias@sst.com.br>
-#
-sub cyclades_snmp {
-   my ($oid, $ret);
-   local $_;
-   
-   $oid = ".1.3.6.1.4.1.2925.3.3.6.1.1.2";
-
-   if (-e $snmpwalk && -x $snmpwalk) { 	
-      #
-      # Get a list of logged in users via snmpwalk
-      #
-      # I'm looking for a way to do this using SNMP_Session
-      # module. Any help would be great.
-      #
-      $_ = `$snmpwalk $ARGV[1] $ARGV[5] $oid`;
-      $ret = ((/$ARGV[3]/) ? 1 : 0);
-   } else {
-      print LOG "   Error: can't execute $snmpwalk\n" if $debug;
-      $ret = 2;
+   while(<FD>) {
+      if (/$ARGV[3]/) {
+	close FD;
+	return 1;
+      }
    }
-   $ret;
+   close FD;
+   0;
 }
 
 #
@@ -767,11 +674,7 @@ sub usrhiper_snmp {
 	my ($login);
 
 	$login = snmpget($ARGV[1], $ARGV[5], "$usrm.4.10.1.1.18.$oidext");
-	#($login) = /^.*\"([^"]+)".*$/;
-	if ($login =~ /\"/) {
-		$login =~ /^.*\"([^"]+)\"/;
-		$login = $1;
-	}
+	($login) = /^.*\"([^"]+)".*$/;
 
 	print LOG "  user at port S$ARGV[2]: $login\n" if ($debug);
 
@@ -876,8 +779,11 @@ sub versanet_snmp {
 	$slot = (($ARGV[2]-$port)/8)+1;
 	print LOG "slot = $slot" if ($debug);
 	$loginname = snmpget($ARGV[1], $ARGV[5], "$vsm.27.1.1.3.$slot.$port");
-
-	print LOG "  user at slot $slot port $port: $loginname\n" if ($debug);	  ($loginname eq $ARGV[3]) ? 1 : 0;	
+#
+#	Note: the $ARGV[5] above could be replaced by the public
+#	      community string defined in Versanet VN2001/VN2002.
+#
+	  print LOG "  user at slot $slot port $port: $loginname\n" if ($debug);	  ($loginname eq $ARGV[3]) ? 1 : 0;	
 }
 
 
@@ -889,9 +795,9 @@ if ($debug) {
 
 if ($#ARGV != 5) {
 	print LOG "Usage: checkrad nas_type nas_ip " .
-			"nas_port login session_id snmp_community\n" if ($debug);
+			"nas_port login session_id community\n" if ($debug);
 	print STDERR "Usage: checkrad nas_type nas_ip " .
-			"nas_port login session_id snmp_community\n";
+			"nas_port login session_id community\n";
 	close LOG if ($debug);
 	exit(2);
 }
@@ -906,20 +812,12 @@ if ($ARGV[0] eq 'livingston') {
 	$ret = &computone_finger;
 } elsif ($ARGV[0] eq 'max40xx') {
 	$ret = &max40xx_finger;
-} elsif ($ARGV[0] eq 'max40xx_snmp') {
-	$ret = &max40xx_snmp;
-} elsif ($ARGV[0] eq 'ascend') {
-	$ret = &ascend_snmp;
 } elsif ($ARGV[0] eq 'portslave') {
 	$ret = &portslave_finger;
 } elsif ($ARGV[0] eq 'tc') {
 	$ret = &tc_tccheck;
 } elsif ($ARGV[0] eq 'pathras') {
-	$ret = &cyclades_telnet;
-} elsif ($ARGV[0] eq 'pr3000') {
-	$ret = &cyclades_snmp;
-} elsif ($ARGV[0] eq 'pr4000') {
-	$ret = &cyclades_snmp;
+	$ret = &pathras_telnet;
 } elsif ($ARGV[0] eq 'patton') {
 	$ret = &patton_snmp;
 } elsif ($ARGV[0] eq 'digitro') {
