@@ -4,8 +4,8 @@
 #define RADACCT_DIR		"/var/log/radacct"
 #define RADLOG_DIR		"/var/log"
 
-#ifndef USEMYSQL
 #define RADIUS_DICTIONARY	"dictionary"
+#ifndef USEMYSQL
 #define RADIUS_CLIENTS		"clients"
 #define RADIUS_USERS		"users"
 #endif
@@ -44,17 +44,23 @@
 #define ASCEND_PORT_HACK
 #define ASCEND_CHANNELS_PER_LINE        23
 
-/* MySQL Stuff */
+#ifdef USEMYSQL
 #include <mysql/mysql.h>
 #define	QUERYLOG	"/var/log/radacct/radius.sql"
 #define	MYSQLCONFIG	"radius.conf"
 #define	MYSQLBACKUP	"/var/log/radacct/sqlbackup.dat"
 #define SQLBIGREC       32
 #define SQLLILREC       15
-
+#define MAX_COMMUNITY_LEN 50
+#endif
 
 /*
  *	Hack for USR gear - uses a different Vendor-Specific attribute
  *	packet layout, argh.
  */
 #define ATTRIB_NMC
+
+/*
+ *    Hack to enable use of 1.5.4.3 dictionaries.
+ */
+#define COMPAT_1543
