@@ -28,7 +28,7 @@
 
 char files_sccsid[] =
 "@(#)users.c	1.12 Copyright 1992 Livingston Enterprises Inc\n"
-"@(#)files.c	2.39 Copyright 1999 Cistron Internet Services B.V.";
+"@(#)files.c	2.42 Copyright 1999 Cistron Internet Services B.V.";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -271,7 +271,7 @@ void presuf_setup(VALUE_PAIR *request_pairs)
 		if ((presuf_pair = pairfind(request_pairs, PW_PREFIX)) == NULL &&
 		    (presuf_pair = pairfind(request_pairs, PW_SUFFIX)) == NULL)
 			return;
-		if (presufcmp(presuf_pair, name_pair->strvalue, name) != 0)
+		if (presufcmp(presuf_pair, name_pair->strvalue, name, sizeof(name)) != 0)
 			return;
 		/*
 		 *	See if username must be stripped.
@@ -296,7 +296,7 @@ void presuf_setup(VALUE_PAIR *request_pairs)
 		if ((presuf_pair = pairfind(pl->check, PW_PREFIX)) == NULL &&
 		    (presuf_pair = pairfind(pl->check, PW_SUFFIX)) == NULL)
 			continue;
-		if (presufcmp(presuf_pair, name_pair->strvalue, name) != 0)
+		if (presufcmp(presuf_pair, name_pair->strvalue, name, sizeof(name)) != 0)
 			continue;
 		/*
 		 *	See if username must be stripped.
@@ -378,35 +378,53 @@ static int groupcmp(VALUE_PAIR *check, char *username)
 /*
  *	Compare prefix/suffix.
  */
-int presufcmp(VALUE_PAIR *check, char *name, char *rest)
+int presufcmp(VALUE_PAIR *check, char *name, char *rest, int restlen)
 {
-	int len, namelen;
-	int ret = -1;
-
+        char *realm;
+        int len, namelen;
+        int ret = -1;
+ 
 #if 0 /* DEBUG */
-	printf("Comparing %s and %s, check->attr is %d\n",
-		name, check->strvalue, check->attribute);
+        printf("Comparing %s and %s, check->attr is %d\n",
+                name, check->strvalue, check->attribute);
 #endif
-
-	len = strlen(check->strvalue);
-	switch (check->attribute) {
-		case PW_PREFIX:
-			ret = strncmp(name, check->strvalue, len);
-			if (ret == 0 && rest)
-				strcpy(rest, name + len);
-			break;
-		case PW_SUFFIX:
-			namelen = strlen(name);
-			if (namelen < len)
-				break;
-			ret = strcmp(name + namelen - len, check->strvalue);
-			if (ret == 0 && rest) {
-				strncpy(rest, name, namelen - len);
-				rest[namelen - len] = 0;
-			}
-			break;
-	}
-	return ret;
+ 
+        /*
+         *      Temporarily strip off the realm, unless ofcourse
+         *      we explicitly want to compare against a realm.
+         */
+        realm = NULL;
+        if (strchr(check->strvalue, '@') == NULL) {
+                realm = strchr(name, '@');
+                if (realm) *realm = 0;
+        }
+ 
+        len = strlen(check->strvalue);
+        switch (check->attribute) {
+                case PW_PREFIX:
+                        ret = strncmp(name, check->strvalue, len);
+                        if (ret == 0 && rest)
+                                strNcpy(rest, name + len, restlen);
+                        break;
+                case PW_SUFFIX:
+                        namelen = strlen(name);
+                        if (namelen < len)
+                                break;
+                        ret = strcmp(name + namelen - len, check->strvalue);
+                        if (ret == 0 && rest) {
+                                strncpy(rest, name, namelen - len);
+                                rest[namelen - len] = 0;
+                        }
+                        break;
+        }
+ 
+        if (realm) *realm = '@';
+        if (ret == 0 && rest && realm) {
+                len = strlen(rest);
+                strNcpy(rest + len, realm, restlen -len);
+        }
+ 
+        return ret;
 }
 
 
@@ -455,6 +473,8 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 					if (auth_item->attribute  !=
 					    PW_USER_NAME)
 						continue;
+					/* Sizes are the same */
+					strcpy(username, auth_item->strvalue);
 				case PW_HUNTGROUP_NAME:
 					break;
 				case PW_HINT:
@@ -476,7 +496,6 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 			result = -1;
 			continue;
 		}
-		strcpy(username, auth_item->strvalue);
 
 		/*
 		 *	OK it is present now compare them.
@@ -488,7 +507,8 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 				if (check_item->attribute == PW_PREFIX ||
 				    check_item->attribute == PW_SUFFIX) {
 					if (presufcmp(check_item,
-					    auth_item->strvalue, username) != 0)
+					    auth_item->strvalue, username,
+					    sizeof(username)) != 0)
 						return -1;
 				}
 				else
@@ -770,7 +790,30 @@ void auth_type_fixup(VALUE_PAIR *check)
 
 }
 
-	
+#ifndef USEMYSQL
+static void pair_sanity(char *fn, PAIR_LIST *entry)
+{
+      VALUE_PAIR *pair;
+
+      for (entry = users; entry; entry = entry->next) {
+              for (pair = entry->reply; pair; pair = pair->next) {
+                      if ((pair->attribute == PW_PASSWORD) ||
+                          (pair->attribute == PW_CHAP_PASSWORD) ||
+                          (pair->attribute >= 1000 &&
+                           pair->attribute <= 65536)) {
+
+              log(L_ERR|L_CONS, "\n[%s:%d] WARNING: Check item \"%s\"\n"
+                  "\tfound in reply item list for user \"%s\".\n"
+                  "\tThis attribute MUST go on the first line"
+                  " with the other check items.\n", fn, entry->lineno,
+                  pair->name, entry->name);
+
+                      }
+              }
+      }
+}	
+#endif
+
 #define FIND_MODE_NAME	0
 #define FIND_MODE_REPLY	1
 
@@ -895,6 +938,10 @@ parse_again:
 					pl = t;
 				last = t;
 
+#ifndef USEMYSQL
+				pair_sanity(file, t);
+#endif
+
 				mode = FIND_MODE_NAME;
 				if (buffer[0] != 0)
 					goto parse_again;
@@ -1094,11 +1141,18 @@ int user_find(char *name, VALUE_PAIR *request_pairs,
 
 #ifdef USEMYSQL
 
-	if ((found = mysql_getvpdata(mysql_authcheck_table, &check_tmp, name, PW_VP_USERDATA)) <= 0)
-		return -1;
-	mysql_getvpdata(mysql_groupcheck_table, &check_tmp, name, PW_VP_GROUPDATA);
-	mysql_getvpdata(mysql_authreply_table, &reply_tmp, name, PW_VP_USERDATA);
-	mysql_getvpdata(mysql_groupreply_table, &reply_tmp, name, PW_VP_GROUPDATA);
+	if ((found = mysql_getvpdata(mysql_authcheck_table, &check_tmp, name, PW_VP_USERDATA)) > 0) {
+		mysql_getvpdata(mysql_groupcheck_table, &check_tmp, name, PW_VP_GROUPDATA);
+		mysql_getvpdata(mysql_authreply_table, &reply_tmp, name, PW_VP_USERDATA);
+		mysql_getvpdata(mysql_groupreply_table, &reply_tmp, name, PW_VP_GROUPDATA);
+	} else {
+
+		int gcheck, greply;
+		gcheck = mysql_getvpdata(mysql_groupcheck_table, &check_tmp, "DEFAULT", PW_VP_GROUPDATA);
+		greply = mysql_getvpdata(mysql_groupreply_table, &reply_tmp, "DEFAULT", PW_VP_GROUPDATA);
+		if (gcheck == 0 || greply == 0)
+			return -1;
+	}
 
 	pairmove(reply_pairs, &reply_tmp);
 	pairmove(check_pairs, &check_tmp);
@@ -1120,7 +1174,7 @@ int user_find(char *name, VALUE_PAIR *request_pairs,
 		/*
 		 *	FIXME: No Prefix / Suffix support for DBM.
 		 */
-		sprintf(buffer, "%s/%s", radius_dir, RADIUS_USERS);
+		sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_USERS);
 #ifdef DBM
 		if (dbminit(buffer) != 0)
 #endif
@@ -1356,6 +1410,7 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 				log(L_CONS|L_ERR, "no memory");
 				exit(1);
 			}
+			/* No need for strncpy - same size */
 			strcpy(pair->name, attr->name);
 			pair->attribute = attr->value;
 			pair->type = attr->type;
@@ -1363,7 +1418,10 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 			switch(pair->type) {
 
 			case PW_TYPE_STRING:
-				strcpy(pair->strvalue, valstr);
+do_copy:
+                                strncpy(pair->strvalue, valstr,
+                                        sizeof(pair->strvalue));
+                                pair->strvalue[sizeof(pair->strvalue) - 1] = 0;
 				pair->length = strlen(pair->strvalue);
 				break;
 
@@ -1375,12 +1433,7 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 				if (attr->value == PW_NAS_PORT_ID) {
 					for(s = valstr; *s; s++)
 						if (!isdigit(*s)) break;
-					if (*s) {
-						pair->type = PW_TYPE_STRING;
-						strcpy(pair->strvalue, valstr);
-						pair->length = strlen(pair->strvalue);
-						break;
-					}
+					if (*s) goto do_copy;
 				}
 				if (isdigit(*valstr)) {
 					pair->lvalue = atoi(valstr);
@@ -1474,7 +1527,7 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
  *	Match a username with a wildcard expression.
  *	Is very limited for now.
  */
-static int matches(char *name, PAIR_LIST *pl, char *matchpart)
+static int matches(char *name, PAIR_LIST *pl, char *matchpart, int matchlen)
 {
 	int len, wlen;
 	int ret = 0;
@@ -1494,11 +1547,11 @@ static int matches(char *name, PAIR_LIST *pl, char *matchpart)
 
 #ifdef USEMYSQL
 		if (strcmp(pl->name, name) == 0)
-			return !presufcmp(tmp, name, matchpart);
+			return !presufcmp(tmp, name, matchpart, matchlen);
 #else
 		if (strncmp(pl->name, "DEFAULT", 7) == 0 ||
 		    strcmp(pl->name, name) == 0)
-			return !presufcmp(tmp, name, matchpart);
+			return !presufcmp(tmp, name, matchpart, matchlen);
 #endif
 	}
 
@@ -1508,13 +1561,13 @@ static int matches(char *name, PAIR_LIST *pl, char *matchpart)
 #ifdef USEMYSQL
 	if (strchr(pl->name, '*') == NULL &&
 	    strcmp(pl->name, name) == 0) {
-		strcpy(matchpart, name);
+		strNcpy(matchpart, name, matchlen);
 		return 1;
 #else
 	if (strchr(pl->name, '*') == NULL &&
 	    (strncmp(pl->name, "DEFAULT", 7) == 0 ||
 	     strcmp(pl->name, name) == 0)) {
-		strcpy(matchpart, name);
+		strNcpy(matchpart, name, matchlen);
 		return 1;
 #endif
 	}
@@ -1522,6 +1575,8 @@ static int matches(char *name, PAIR_LIST *pl, char *matchpart)
 	/*
 	 *	Normally, we should return 0 here, but we
 	 *	support the old * stuff.
+         *      FIXME: this doesn't support realsm yet, while
+         *      presufcmp does
 	 */
 	len = strlen(name);
 	wlen = strlen(wild);
@@ -1532,13 +1587,13 @@ static int matches(char *name, PAIR_LIST *pl, char *matchpart)
 		wild++;
 		wlen--;
 		if (wlen <= len && strcmp(name + (len - wlen), wild) == 0) {
-			strcpy(matchpart, name);
+			strNcpy(matchpart, name, matchlen);
 			matchpart[len - wlen] = 0;
 			ret = 1;
 		}
 	} else if (wild[wlen - 1] == '*') {
 		if (wlen <= len && strncmp(name, wild, wlen - 1) == 0) {
-			strcpy(matchpart, name + wlen - 1);
+			strNcpy(matchpart, name + wlen - 1, matchlen);
 			ret = 1;
 		}
 	}
@@ -1587,6 +1642,7 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	/*
 	 *	Windows NT machines often authenticate themselves as
 	 *	NT_DOMAIN\username. Try to be smart about this.
+	 *	Note that sizeof(newname) == sizeof(name) 
 	 *
 	 *	FIXME: should we handle this as a REALM ?
 	 */
@@ -1630,7 +1686,7 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	}
 
 	for (i = hints; i; i = i->next) {
-		if (matches(name, i, newname)) {
+		if (matches(name, i, newname, sizeof(newname))) {
 			DEBUG2("  hints: Matched %s at %d",
 			       i->name, i->lineno);
 			break;
@@ -1659,6 +1715,7 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	if (do_strip) {
 		tmp = pairfind(request_pairs, PW_USER_NAME);
 		if (tmp) {
+			/* Same size */
 			strcpy(tmp->strvalue, newname);
 			tmp->length = strlen(tmp->strvalue);
 		}
@@ -1796,7 +1853,7 @@ int read_clients_file(char *file)
 			continue;
 
 		shortnm[0] = 0;
-		if (sscanf(buffer, "%s%s%s", hostnm, secret, shortnm) < 2) {
+		if (sscanf(buffer, "%127s%31s%31s", hostnm, secret, shortnm) < 2) {
 			log(L_ERR, "%s[%d]: syntax error", file, lineno);
 			continue;
 		}
@@ -1809,9 +1866,10 @@ int read_clients_file(char *file)
 		}
 
 		c->ipaddr = get_ipaddr(hostnm);
-		strcpy(c->secret, secret);
-		strcpy(c->shortname, shortnm);
-		strcpy(c->longname, ip_hostname(c->ipaddr));
+		strNcpy(c->secret, secret, sizeof(c->secret));
+		strNcpy(c->shortname, shortnm, sizeof(c->shortname));
+		strNcpy(c->longname, ip_hostname(c->ipaddr),
+			sizeof(c->longname));
 
 		c->next = clients;
 		clients = c;
@@ -1908,7 +1966,7 @@ int read_naslist_file(char *file)
 		if (buffer[0] == '#' || buffer[0] == '\n')
 			continue;
 		nastype[0] = 0;
-		if (sscanf(buffer, "%s%s%s", hostnm, shortnm, nastype) < 2) {
+		if (sscanf(buffer, "%127s%31s%31s", hostnm, shortnm, nastype) < 2) {
 			log(L_ERR, "%s[%d]: syntax error", file, lineno);
 			continue;
 		}
@@ -1919,9 +1977,10 @@ int read_naslist_file(char *file)
 		}
 
 		c->ipaddr = get_ipaddr(hostnm);
-		strcpy(c->nastype, nastype);
-		strcpy(c->shortname, shortnm);
-		strcpy(c->longname, ip_hostname(c->ipaddr));
+		strNcpy(c->nastype, nastype, sizeof(c->nastype));
+		strNcpy(c->shortname, shortnm, sizeof(c->shortname));
+		strNcpy(c->longname, ip_hostname(c->ipaddr),
+			sizeof(c->longname));
 
 		c->next = naslist;
 		naslist = c;
@@ -1941,7 +2000,7 @@ int read_realms_file(char *file)
 	char	buffer[256];
 	char	realm[32];
 	char	hostnm[128];
-	char	opts[32];
+	char	opts1[32], opts2[32];
 	char	*p;
 	int	lineno = 0;
 	REALM	*c;
@@ -1961,8 +2020,9 @@ int read_realms_file(char *file)
 		lineno++;
 		if (buffer[0] == '#' || buffer[0] == '\n')
 			continue;
-		opts[0] = 0;
-		if (sscanf(buffer, "%s%s%s", realm, hostnm, opts) < 2) {
+		opts1[0] = opts2[0] = 0;
+		if (sscanf(buffer, "%31s%127s%31s%31s",
+		     realm, hostnm, opts1, opts2) < 2) {
 			log(L_ERR, "%s[%d]: syntax error", file, lineno);
 			continue;
 		}
@@ -1982,9 +2042,13 @@ int read_realms_file(char *file)
 		}
 		if (strcmp(hostnm, "LOCAL") != 0)
 			c->ipaddr = get_ipaddr(hostnm);
-		strcpy(c->realm, realm);
-		strcpy(c->server, hostnm);
-		c->striprealm = strcmp(opts, "nostrip") != 0;
+		strNcpy(c->realm, realm, sizeof(c->realm));
+		strNcpy(c->server, hostnm, sizeof(c->server));
+		/* Yes we could do this more intelligently */
+                c->striprealm = strcmp(opts1, "nostrip") != 0 ||
+                                strcmp(opts2, "nostrip") != 0;
+                c->dohints    = strcmp(opts1, "hints") == 0 ||
+                                strcmp(opts2, "hints") == 0;
 		c->next = realms;
 		realms = c;
 	}
@@ -2098,8 +2162,7 @@ static int checkdbm(char *users, char *ext)
 	char buffer[256];
 	struct stat st;
 
-	strcpy(buffer, users);
-	strcat(buffer, ext);
+	sprintf(buffer, "%.200s%.50s", users, ext);
 
 	return stat(buffer, &st);
 }
@@ -2115,7 +2178,7 @@ int read_config_files()
 
 #ifndef USEMYSQL
 	pairlist_free(&users);
-	sprintf(buffer, "%s/%s", radius_dir, RADIUS_USERS);
+	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_USERS);
 #endif
 	pairlist_free(&huntgroups);
 	pairlist_free(&hints);
@@ -2131,30 +2194,10 @@ int read_config_files()
 	if (!use_dbm && (users = file_read(buffer, 1)) == NULL)
 		return -1;
 
-        /*
-         *  Double check the 'users' file for sanity
-         */
-        if (!use_dbm) {
-          PAIR_LIST *entry;
-          VALUE_PAIR *pair;
-
-          for (entry = users; entry; entry = entry->next) {
-            for (pair = entry->reply; pair; pair = pair->next) {                            if ((pair->attribute == PW_PASSWORD) ||
-                  (pair->attribute == PW_CHAP_PASSWORD) ||
-                  (pair->attribute >= 1000)) {
-                log(L_ERR|L_CONS, "\n[%s:%d] WARNING: Check item \"%s\"\n"
-                    "\tfound in reply item list for user \"%s\".\n"
-                    "\tThis attribute MUST go on the first line"
-                    " with the other check items.\n", buffer, entry->lineno,
-                    pair->name, entry->name);
-              }
-            }
-          }
-        }
 #endif
-	sprintf(buffer, "%s/%s", radius_dir, RADIUS_HUNTGROUPS);
+	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_HUNTGROUPS);
 	huntgroups = file_read(buffer, 0);
-	sprintf(buffer, "%s/%s", radius_dir, RADIUS_HINTS);
+	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_HINTS);
 	hints      = file_read(buffer, 0);
 #ifdef USEMYSQL
 	if (mysql_read_realms(mysql_realm_table) < 0)
@@ -2164,14 +2207,14 @@ int read_config_files()
 	if (mysql_read_clients(mysql_nas_table) < 0)
 		return -1;
 #else
-	sprintf(buffer, "%s/%s", radius_dir, RADIUS_CLIENTS);
+	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_CLIENTS);
 	if (read_clients_file(buffer) < 0)
 		return -1;
-	sprintf(buffer, "%s/%s", radius_dir, RADIUS_NASLIST);
+	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_NASLIST);
 	if (read_naslist_file(buffer) < 0)
 		return -1;
 
-	sprintf(buffer, "%s/%s", radius_dir, RADIUS_REALMS);
+	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_REALMS);
 	if (read_realms_file(buffer) < 0)
 		return -1;
 #endif
