@@ -41,7 +41,7 @@ int mysql_start ()
 	FILE    *sqlfd;
         char    dummystr[64];
         char    namestr[64];
-        int     line_no;
+        int     line_no = 0;
         char    buffer[256];
         char    sqlfile[256];
 	MyAuthSock = NULL;
@@ -63,6 +63,7 @@ int mysql_start ()
        strcpy(mysql_dict_table,"dictionary");
        sqltrace = 0;
        mysql_keepopen = 0;
+       sensitiveusername = 1;
 
         sprintf(sqlfile, "%s/%s", radius_dir, MYSQLCONFIG);
         if((sqlfd = fopen(sqlfile, "r")) == (FILE *)NULL) {
@@ -215,6 +216,18 @@ int mysql_start ()
                         }
                        }
                }
+               if(strncasecmp(buffer, "sensitiveusername", 17) == 0) {
+                        /* Read the SENSITIVEUSERNAME line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of sqlserver file %s", line_no,sqlfile);
+                       } else {
+                         if(strncasecmp(namestr, "on", 2) == 0) {
+                           sensitiveusername = 1;
+                        } else {
+                           sensitiveusername = 0;
+                        }
+                       }
+               }
 
        }
        fclose(sqlfd);
@@ -674,6 +687,7 @@ int mysql_checksocket(const char *facility) {
 int mysql_getvpdata(char *table, VALUE_PAIR **vp, char *user, int mode) {
 
 	char		querystr[256];
+	char		authstr[256];
 	char		username[AUTH_STRING_LEN*2+1];
 	MYSQL_RES	*result;
 	MYSQL_ROW	row;
@@ -688,11 +702,19 @@ int mysql_getvpdata(char *table, VALUE_PAIR **vp, char *user, int mode) {
 	/* FIXME CHECK user for weird charactors!! */
 	mysql_escape_string(username, user, length);
 
-	if (mode == PW_VP_USERDATA)
-		sprintf(querystr, "SELECT * FROM %s WHERE UserName = '%s'", table, username);
-	else if (mode == PW_VP_GROUPDATA)
-		sprintf(querystr, "SELECT %s.* FROM %s, %s WHERE %s.UserName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, mysql_usergroup_table, mysql_usergroup_table, username, mysql_usergroup_table, table, table);
-	else if (mode == PW_VP_REALMDATA)
+	if (mode == PW_VP_USERDATA) {
+		if (sensitiveusername) 
+			sprintf(authstr, "STRCMP(Username, '%s') = 0", username);
+		else
+			sprintf(authstr, "UserName = '%s'", username);
+		sprintf(querystr, "SELECT * FROM %s WHERE %s", table, authstr);
+	} else if (mode == PW_VP_GROUPDATA) {
+		if (sensitiveusername) 
+			sprintf(authstr, "STRCMP(%s.Username, '%s') = 0", mysql_usergroup_table, username);
+		else
+			sprintf(authstr, "%s.UserName = '%s'",mysql_usergroup_table, username);
+		sprintf(querystr, "SELECT %s.* FROM %s, %s WHERE %s AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, mysql_usergroup_table, authstr, mysql_usergroup_table, table, table);
+	} else if (mode == PW_VP_REALMDATA)
 		sprintf(querystr, "SELECT %s.* FROM %s, %s WHERE %s.RealmName = '%s' AND %s.GroupName = %s.GroupName ORDER BY %s.id", table, table, mysql_realmgroup_table, mysql_realmgroup_table, username, mysql_realmgroup_table, table, table);
 	if (sqltrace)
 		DEBUG(querystr);
@@ -827,6 +849,7 @@ static int mysql_check_ts(MYSQL_ROW row) {
 int mysql_check_multi(char *name, VALUE_PAIR *request, int maxsimul) {
 
 	char		querystr[256];
+	char		authstr[256];
 	VALUE_PAIR	*fra;
 	MYSQL_RES	*result;
 	MYSQL_ROW	row;
@@ -836,7 +859,11 @@ int mysql_check_multi(char *name, VALUE_PAIR *request, int maxsimul) {
 
 	if (!mysql_checksocket("Auth"))
 		return 0;
-	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE UserName = '%s' AND AcctStopTime = 0", mysql_acct_table, name);
+	if (sensitiveusername)
+		sprintf(authstr, "STRCMP(UserName, '%s') = 0", name);
+	else
+		sprintf(authstr, "UserName = '%s'", name);
+	sprintf(querystr, "SELECT COUNT(*) FROM %s WHERE %s AND AcctStopTime = 0", mysql_acct_table, authstr);
 	if (sqltrace)
 		DEBUG(querystr);
 	mysql_query(MyAuthSock, querystr);
@@ -933,15 +960,22 @@ int mysql_read_naslist(char *nastable)
 				return -1;
 			}
 
+			c->longname[0] = 0;
+			c->shortname[0] = 0;
+			c->community[0] = 0;
+			if (row[1] != NULL)
+                		strNcpy(c->longname, row[1], sizeof(c->longname));
+			if (row[2] != NULL)
+                		strNcpy(c->shortname, row[2], sizeof(c->shortname));
                 	c->ipaddr = ipstr2long(row[3]);
-                	strcpy(c->nastype, row[4]);
-                	strcpy(c->shortname, row[2]);
-                	strcpy(c->longname, row[1]);
-			strncpy(c->community, row[7], MAX_COMMUNITY_LEN);
-			if (strcmp(row[8], "on") == 0 || strcmp(row[8], "ON") == 0)
-				c->usesnmp = 1;
-			else 
-				c->usesnmp = 0; 
+			if (row[4] != NULL)
+	                	strNcpy(c->nastype, row[4], sizeof(c->nastype));
+			if (row[7] != NULL) {
+				strncpy(c->community, row[7], MAX_COMMUNITY_LEN);
+			}
+
+			c->usesnmp = strcmp(row[8], "ON") == 0 ||
+				     strcmp(row[8], "on") == 0;
 
         	        c->next = naslist;
                 	naslist = c;
@@ -984,10 +1018,16 @@ int mysql_read_clients(char *nastable)
 				return -1;
 			}
 
+			c->secret[0] = 0;
+			c->shortname[0] = 0;
+			c->longname[0] = 0;
                 	c->ipaddr = ipstr2long(row[3]);
-                	strcpy(c->secret, row[6]);
-                	strcpy(c->shortname, row[2]);
-                	strcpy(c->longname, row[1]);
+			if (row[1] != NULL)
+                		strNcpy(c->longname, row[1], sizeof(c->longname));
+			if (row[2] != NULL)
+                		strNcpy(c->shortname, row[2], sizeof(c->shortname));
+			if (row[6] != NULL)
+	                	strNcpy(c->secret, row[6], sizeof(c->secret));
 	
         	        c->next = clients;
                 	clients = c;
@@ -1040,7 +1080,7 @@ int mysql_dict_init(void) {
   		MyAuthSock = NULL;
 	} else {
 		while ((row = mysql_fetch_row(result))) {
-			line_no = atoi(row[0]);
+			line_no = strtol(row[0], NULL, 0);
 	
 			is_attrib = 0;
 			if (strncmp(row[1], "ATTRIBUTE", 9) == 0)
@@ -1241,6 +1281,8 @@ int mysql_read_realms(char *realmtable) {
    char		querystr[2048];
    MYSQL_RES	*result;
    MYSQL_ROW	row;
+   char opts1[32];
+   char opts2[32];
    
    realm_free(realms);
    realms = NULL;
@@ -1258,43 +1300,55 @@ int mysql_read_realms(char *realmtable) {
    mysql_query(MyAuthSock, querystr);
    
    if (!(result = mysql_store_result(MyAuthSock)) && 
-       mysql_num_fields(MyAuthSock)) 
-     {
+       mysql_num_fields(MyAuthSock)) {
        log(L_ERR,"MYSQL Error: Cannot get result");
        log(L_ERR,"MYSQL error: %s",mysql_error(MyAuthSock));
        mysql_close(MyAuthSock);
        MyAuthSock = NULL;
-     } 
-   else 
-     {
-       REALM   *c;
+   } else {
+
+	REALM   *c;
        
-       while ((row = mysql_fetch_row(result))) 
-       {
-         if ((c = malloc(sizeof(REALM))) == NULL) 
-           {
-             log(L_CONS|L_ERR, "out of memory");
-             return -1;
-           }
-        
-	strcpy(c->realm, row[0]);
-	strcpy(c->server, row[1]);
-	c->ipaddr = ipstr2long(row[2]);
-	if (strcmp(row[3],"0") == 0)
-		c->auth_port = auth_port;
-	else
-		c->auth_port = strtol(row[3], NULL, 0);
-	c->acct_port = c->auth_port + 1;
-	c->striprealm = strcmp(row[4], "nostrip") != 0;
- 
-         c->next = realms;
-         realms = c;
-       }
+	while ((row = mysql_fetch_row(result))) {
+		if ((c = malloc(sizeof(REALM))) == NULL) {
+			log(L_CONS|L_ERR, "out of memory");
+			return -1;
+		}
+
+		opts1[0] = opts2[0] = 0;
+		c->realm[0] = 0;
+		c->server[0] = 0;
+
+		if (c->realm != NULL)
+			strNcpy(c->realm, row[0], sizeof(c->realm));
+		if (c->server != NULL)
+			strNcpy(c->server, row[1], sizeof(c->server));
+		if (strcmp(row[1], "LOCAL") != 0)
+			c->ipaddr = get_ipaddr(row[1]);
+		else
+			c->ipaddr = ipstr2long(row[2]);
+		if (strcmp(row[3],"0") == 0)
+			c->auth_port = auth_port;
+		else
+			c->auth_port = strtol(row[3], NULL, 0);
+		c->acct_port = c->auth_port + 1;
+		if (row[4] != NULL) {
+			sscanf(row[4], "%31s%31s", opts1, opts2);
+
+			c->striprealm = strcmp(opts1, "nostrip") != 0 ||
+					strcmp(opts2, "nostrip") != 0;
+			c->dohints    = strcmp(opts1, "hints") == 0 ||
+					strcmp(opts2, "hints") == 0;
+	 	}
+
+		c->next = realms;
+		realms = c;
+       	}
        
-       mysql_free_result(result);
-     }
+	mysql_free_result(result);
+   }
    return 0;
- }
+}
 
 
 /* ENDIF from top */
