@@ -84,6 +84,7 @@ static int		request_list_busy = 0;
 static int		sockfd;
 static int		acctfd;
 static int		spawn_flag;
+static int		fork_auth;
 static int		acct_pid;
 static int		radius_pid;
 static int		need_reload = 0;
@@ -196,6 +197,7 @@ int main(int argc, char **argv)
 
 	debug_flag = 0;
 	spawn_flag = 1;
+	fork_auth = 0;
 	radacct_dir = RADACCT_DIR;
 	radlog_dir = RADLOG_DIR;
 	radius_dir = RADIUS_DIR;
@@ -222,7 +224,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Process the options.
 	 */
-	while((argval = getopt(argc, argv, "ASa:ci:l:d:bfp:svxyz")) != EOF) {
+	while((argval = getopt(argc, argv, "ASa:ci:l:d:bfp:snvxyz")) != EOF) {
 
 		switch(argval) {
 
@@ -274,6 +276,9 @@ int main(int argc, char **argv)
 		case 's':	/* Single process mode */
 			spawn_flag = 0;
 			break;
+		case 'n':  /* Fork individual authentication requests (normal) but MySQL code does not like this*/
+			fork_auth = 1;
+			break;
 
 		case 'v':
 			version();
@@ -550,7 +555,7 @@ int radrespond(AUTH_REQ *authreq, int activefd)
 	switch(authreq->code) {
 
 	case PW_AUTHENTICATION_REQUEST:
-		dospawn = spawn_flag;
+		dospawn = fork_auth;
 		fun = rad_authenticate;
 		break;
 	
