@@ -118,6 +118,7 @@ typedef struct realm {
 	int			auth_port;
 	int			acct_port;
 	int			striprealm;
+	int			dohints;
 	struct realm		*next;
 } REALM;
 
@@ -271,6 +272,7 @@ void		authfree(AUTH_REQ *authreq);
 void		(*sun_signal(int signo, void (*func)(int)))(int);
 #define signal sun_signal
 #endif
+char		*strNcpy(char *dest, char *src, int n);
 
 /* radius.c */
 int		rad_send_reply(int, AUTH_REQ *, VALUE_PAIR *, char *, int);
@@ -295,7 +297,7 @@ char		*nas_name2(AUTH_REQ *r);
 char		*auth_name(AUTH_REQ *authreq, int do_cid);
 int		read_naslist_file(char *);
 int		read_config_files(void);
-int		presufcmp(VALUE_PAIR *check, char *name, char *rest);
+int		presufcmp(VALUE_PAIR *check, char *name, char *rest, int rl);
 void		pairmove(VALUE_PAIR **to, VALUE_PAIR **from);
 void		pairmove2(VALUE_PAIR **to, VALUE_PAIR **from, int attr);
 void		auth_type_fixup(VALUE_PAIR *check);
@@ -352,6 +354,7 @@ char mysql_realmgroup_table[MAX_TABLE_LEN];
 char mysql_nas_table[MAX_TABLE_LEN];
 char mysql_dict_table[MAX_TABLE_LEN];
 int  mysql_keepopen;
+int  sensitiveusername;
 int  sqltrace; 
 MYSQL *MyAuthSock;
 MYSQL *MyAcctSock;
