@@ -37,7 +37,7 @@ char version_sccsid[] =
  *  would like a range of versions allocated for your use.
  */
 
-#define		VERSION		"cistron-1.6.0 14-Sep-1999 +ICRADIUS-0.9"
+#define		VERSION		"cistron-1.6.1 19-Oct-1999 +ICRADIUS 0.10"
 
 /*
  *	Display the revision number for this program
@@ -73,6 +73,9 @@ void version(void)
 #if defined(ATTRIB_NMC)
 	fprintf(stderr, " ATTRIB_NMC");
 #endif
+#if defined(COMPAT_1543)
+	fprintf(stderr, " COMPAT_1543");
+#endif
 
 	/* here are all the system definitions compilation uses */
 #if defined(__alpha)
@@ -103,7 +106,7 @@ void version(void)
 	fprintf(stderr," M_UNIX");
 #endif
 #ifdef USEMYSQL
-	fprintf(stderr, " USEMYSQL");
+	fprintf(stderr," USEMYSQL");
 #endif
 	fprintf(stderr,"\n");
 	exit(0);
